import 'dart:io';
import 'dart:convert';

import 'package:origami_endpoint/models/endpointConnection.dart';
import 'package:origami_endpoint/models/endpointErrorMessage.dart';
import 'package:origami_endpoint/service/sessionManager/sessionManager.dart';
import 'package:origamicore/models/messageModel.dart';
import 'package:origamicore/routing/routeResponse.dart';
import 'package:origamicore/routing/router.dart';

class HttpIndex
{
  EndpointConnection connection = EndpointConnection();
  SessionManager sessionManager;
  HttpIndex({this.connection,this.sessionManager})
  {
    init();
  }
  dynamic init()async
  { 
    //var a=InternetAddress.loopbackIPv4;
    var bindAddress=connection.bindAddress??InternetAddress.loopbackIPv4; 
    var server = await HttpServer.bind(bindAddress, connection.protocol?.port??0);
    print (bindAddress);
    print("Search server is running on  'http://${server.address.address}:${connection.protocol?.port??0}/'");
     await for (var req in server) {
       accepRequest(req);
    }
  }
  void accepRequest(HttpRequest  req)async
  {
       var strResponse='';
       try{
          var resp= await sendRequest(req);
          if(resp.error!=null)
          {
            strResponse=json.encode(resp.error?.toJson()); 
            req.response.statusCode=500;
          }
          else
          {
            strResponse=json.encode(resp.response?.toJson()); 
            req.response.statusCode=200;

          }
       }catch(exp)
       {
          strResponse=exp.toString();
       }
      req.response.headers.add('Access-Control-Allow-Origin', '*');
      req.response.headers.add('Access-Control-Allow-Methods', 'POST,GET,DELETE,PUT,OPTIONS');
      req.response.write(strResponse);
      await req.response.close();

  }
  Future<RouteResponse> sendRequest(HttpRequest request) async
  {
    var body={};
    if(request.method=='POST')
    {
      var dt =await  Utf8Codec().decodeStream(request);
      body = json.decode(dt);
    }
    request.uri.queryParameters.forEach((key, value) { 
      body[key]=value;
    }); 
    var path=request.uri.pathSegments;
    if(path.length<2)
    {
      return  RouteResponse(error: EndpointErrorMessage.connectionNotFound);
    }
    var message= MessageModel(data: body);
    if(sessionManager!=null && request.headers['authorization']!=null)
    {
      message.session=await sessionManager.getSession(request.headers['authorization'][0]);
    }
    // var dt =await UTF8.decodeStream(request);
    var resp=await Router.runExternal(path[0], path[1],message);
    if(resp.session!=null)
    {
      resp.response.session=sessionManager.setSession('token', resp.session);
    }
    return resp;
  }
}